""" Requires Flickr API: http://stuvel.eu/flickrapi
    For Python 3 compatibility use >= 2.0 or tip:
    https://bitbucket.org/sybren/flickrapi
"""
from __future__ import absolute_import, unicode_literals

import flickrapi

FLICKR_API_KEY = '93c7a048ba770bf447664589e59110bf'


class FlickrInteresting(object):
    """ returns todays interesting images.
    """
    def __init__(self, date=None, per_page=100, **kwargs):
        flickr = flickrapi.FlickrAPI(FLICKR_API_KEY)
        self.index = 0
        self.date = date
        if per_page > 500:
            raise AttributeError('Only 500 images per page allowed.')
        self.per_page = kwargs.get('count', per_page)

        photos_dom = flickr.interestingness_getList()[0]
        self.photos = photos_dom.findall('photo')

    def __iter__(self):
        return self

    def next(self):
        if self.index >= self.per_page:
            raise StopIteration
        url = 'http://farm%(farm_id)s.staticflickr.com/%(server_id)s/%(id)s_%(secret)s.jpg' % {
            'farm_id': self.photos[self.index].get('farm'),
            'server_id': self.photos[self.index].get('server'),
            'id': self.photos[self.index].get('id'),
            'secret': self.photos[self.index].get('secret'),
        }
        self.index += 1
        return url



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import random

US_MALE_NAMES = ['Jacob', 'Ethan', 'Michael', 'Alexander', 'William', 'Joshua',
                 'Daniel', 'Jayden', 'Noah', 'Anthony', 'Jonathan', 'David',
                 'John', 'Mark', 'Calvin', 'Jeremy', 'Ethan', 'Phillip',
                 'Brian', 'Isaac', 'Abraham', 'Jesse', 'Lawrence',
                 'Jeffrey', 'Steve', 'Paul', 'Robert', 'Winston', 'Ken',
                 'Caleb', 'George', 'Brent', 'Joseph', 'Ian', 'Peter', 'Luke',
                 'Ted', 'Andrew', 'Joe', 'Dennis', 'Bill', 'Felix', 'Don',
                 'Oliver', 'Harry', 'Samuel', 'Justin', 'Brooks', 'Nathan']
US_FEMALE_NAMES = ['Isabella', 'Emma', 'Olivia', 'Sophia', 'Ava', 'Emily',
                   'Madison', 'Abigail', 'Chloe', 'Mia', 'Alice', 'Helen',
                   'Grace', 'Joanna', 'Ann', 'Lisa', 'Lily,' 'May', 'June',
                   'April', 'Jane', 'Elise', 'Kristy', 'Katie', 'Kathy',
                   'Julie', 'Jamie', 'Carol', 'Carrie', 'Elizabeth', 'Robin',
                   'Sally', 'Jackie', 'Sherry', 'Christine', 'Angela', 'Judy',
                   'Ruth', 'Brooke', 'Megan', 'Dawn', 'Rebecca', 'Esther',
                   'Claire']


class FirstNames(object):
    """ can iterate over names for the given gender.
    """

    def __init__(self, gender=None, male_names=US_MALE_NAMES,
                 female_names=US_FEMALE_NAMES, *args, **kwargs):
        self.gender = gender
        if gender in ['male', 'm']:
            self.first_names = male_names
        elif gender in ['female', 'f']:
            self.first_names = female_names
        else:
            self.first_names = male_names + female_names
            random.shuffle(self.first_names)
        self.index = 0
        self.length = len(self.first_names)

    def __iter__(self):
        return self

    def next(self):
        self.index += 1
        return self.first_names[self.index % self.length]


GERMAN_LAST_NAMES = ['Müller', 'Schmid', 'Schneider', 'Fischer', 'Weber',
                     'Meyer',
                     'Wagner', 'Becker', 'Schutz', 'Hoffmann', 'Schäfer',
                     'Koch', 'Bauer', 'Richter', 'Bächler', 'Kestenholz']

ASIAN_LAST_NAMES = ['Wang', 'Chen', 'Chou', 'Tang', 'Huang', 'Liu', 'Shih',
                    'Su', 'Song',
                    'Lin', 'Yu', 'Yang', 'Chan', 'Tsai', 'Wong', 'Hsu',
                    'Chang', 'Cheng',
                    'Park', 'Kim', 'Choi', 'Kang', 'Hwang']

US_LAST_NAMES = ['Smith', 'Walker', 'Conroy', 'Stevens', 'Jones', 'Armstrong',
                 'Johnson',
                 'White', 'Olson', 'Ellis', 'Mitchell', 'Forrest', 'Baker',
                 'Portman',
                 'Davis', 'Clark', 'Roberts', 'Jackson', 'Marshall', 'Decker',
                 'Brown']


class LastNames(object):
    """ Keeps returning last names
    """

    def __init__(self,
                 last_names=GERMAN_LAST_NAMES + ASIAN_LAST_NAMES + US_LAST_NAMES,
                 *args, **kwargs):
        self.last_names = last_names
        self.index = 0
        self.length = len(self.last_names)

    def __iter__(self):
        return self

    def next(self):
        self.index += 1
        return self.last_names[self.index % self.length]



from __future__ import absolute_import, unicode_literals
import csv
import os
import codecs


class TopUsCities(object):
    """ Returns a name of a US city and state. e.g. "New York, NY".  """

    def __init__(self):
        path = os.path.dirname(os.path.realpath(__file__))
        self.cities = []
        with codecs.open(os.path.join(path, 'US_Top5000Population.csv'),
                         encoding='utf-8') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                self.cities.append('%s, %s' % (row[0].strip(), row[1].strip()))

    def __call__(self):
        return self.cities



from __future__ import absolute_import, unicode_literals
import csv
import os
import codecs

class TopUrl(object):
    """ Returns an URL  """

    def __init__(self, prefix=""):
        path = os.path.dirname(os.path.realpath(__file__))
        self.urls = []
        with codecs.open(os.path.join(path, 'top-10kURL.csv'), encoding='utf-8') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                self.urls.append('%s%s' % (prefix, row[0]))

    def __call__(self):
        return self.urls


# -*- coding: utf-8 -*-
""" taken from http://mdbenoit.com/rtg.htm """
from __future__ import unicode_literals
import random

NOUNS = (
    "Dream", "Dreamer", "Dreams", "Waves",
    "Sword", "Kiss", "Sex", "Lover",
    "Slave", "Slaves", "Pleasure", "Servant",
    "Servants", "Snake", "Soul", "Touch",
    "Men", "Women", "Gift", "Scent",
    "Ice", "Snow", "Night", "Silk", "Secret", "Secrets",
    "Game", "Fire", "Flame", "Flames",
    "Husband", "Wife", "Man", "Woman", "Boy", "Girl",
    "Truth", "Edge", "Boyfriend", "Girlfriend",
    "Body", "Captive", "Male", "Wave", "Predator",
    "Female", "Healer", "Trainer", "Teacher",
    "Hunter", "Obsession", "Hustler", "Consort",
    "Dream", "Dreamer", "Dreams", "Rainbow",
    "Dreaming", "Flight", "Flying", "Soaring",
    "Wings", "Mist", "Sky", "Wind",
    "Winter", "Misty", "River", "Door",
    "Gate", "Cloud", "Fairy", "Dragon",
    "End", "Blade", "Beginning", "Tale",
    "Tales", "Emperor", "Prince", "Princess",
    "Willow", "Birch", "Petals", "Destiny",
    "Theft", "Thief", "Legend", "Prophecy",
    "Spark", "Sparks", "Stream", "Streams", "Waves",
    "Sword", "Darkness", "Swords", "Silence", "Kiss",
    "Butterfly", "Shadow", "Ring", "Rings", "Emerald",
    "Storm", "Storms", "Mists", "World", "Worlds",
    "Alien", "Lord", "Lords", "Ship", "Ships", "Star",
    "Stars", "Force", "Visions", "Vision", "Magic",
    "Wizards", "Wizard", "Heart", "Heat", "Twins",
    "Twilight", "Moon", "Moons", "Planet", "Shores",
    "Pirates", "Courage", "Time", "Academy",
    "School", "Rose", "Roses", "Stone", "Stones",
    "Sorcerer", "Shard", "Shards", "Slave", "Slaves",
    "Servant", "Servants", "Serpent", "Serpents",
    "Snake", "Soul", "Souls", "Savior", "Spirit",
    "Spirits", "Voyage", "Voyages", "Voyager", "Voyagers",
    "Return", "Legacy", "Birth", "Healer", "Healing",
    "Year", "Years", "Death", "Dying", "Luck", "Elves",
    "Tears", "Touch", "Son", "Sons", "Child", "Children",
    "Illusion", "Sliver", "Destruction", "Crying", "Weeping",
    "Gift", "Word", "Words", "Thought", "Thoughts", "Scent",
    "Ice", "Snow", "Night", "Silk", "Guardian", "Angel",
    "Angels", "Secret", "Secrets", "Search", "Eye", "Eyes",
    "Danger", "Game", "Fire", "Flame", "Flames", "Bride",
    "Husband", "Wife", "Time", "Flower", "Flowers",
    "Light", "Lights", "Door", "Doors", "Window", "Windows",
    "Bridge", "Bridges", "Ashes", "Memory", "Thorn",
    "Thorns", "Name", "Names", "Future", "Past",
    "History", "Something", "Nothing", "Someone",
    "Nobody", "Person", "Man", "Woman", "Boy", "Girl",
    "Way", "Mage", "Witch", "Witches", "Lover",
    "Tower", "Valley", "Abyss", "Hunter",
    "Truth", "Edge"
)
ADJECTIVES = (
    "Lost", "Only", "Last", "First",
    "Third", "Sacred", "Bold", "Lovely",
    "Final", "Missing", "Shadowy", "Seventh",
    "Dwindling", "Missing", "Absent",
    "Vacant", "Cold", "Hot", "Burning", "Forgotten",
    "Weeping", "Dying", "Lonely", "Silent",
    "Laughing", "Whispering", "Forgotten", "Smooth",
    "Silken", "Rough", "Frozen", "Wild",
    "Trembling", "Fallen", "Ragged", "Broken",
    "Cracked", "Splintered", "Slithering", "Silky",
    "Wet", "Magnificent", "Luscious", "Swollen",
    "Erect", "Bare", "Naked", "Stripped",
    "Captured", "Stolen", "Sucking", "Licking",
    "Growing", "Kissing", "Green", "Red", "Blue",
    "Azure", "Rising", "Falling", "Elemental",
    "Bound", "Prized", "Obsessed", "Unwilling",
    "Hard", "Eager", "Ravaged", "Sleeping",
    "Wanton", "Professional", "Willing", "Devoted",
    "Misty", "Lost", "Only", "Last", "First",
    "Final", "Missing", "Shadowy", "Seventh",
    "Dark", "Darkest", "Silver", "Silvery", "Living",
    "Black", "White", "Hidden", "Entwined", "Invisible",
    "Next", "Seventh", "Red", "Green", "Blue",
    "Purple", "Grey", "Bloody", "Emerald", "Diamond",
    "Frozen", "Sharp", "Delicious", "Dangerous",
    "Deep", "Twinkling", "Dwindling", "Missing", "Absent",
    "Vacant", "Cold", "Hot", "Burning", "Forgotten",
    "Some", "No", "All", "Every", "Each", "Which", "What",
    "Playful", "Silent", "Weeping", "Dying", "Lonely", "Silent",
    "Laughing", "Whispering", "Forgotten", "Smooth", "Silken",
    "Rough", "Frozen", "Wild", "Trembling", "Fallen",
    "Ragged", "Broken", "Cracked", "Splintered"
)


class Title(object):
    def __iter__(self):
        return self

    def make_title(self):
        adj = random.choice(ADJECTIVES)
        noun = random.choice(NOUNS)
        noun2 = random.choice(NOUNS)
        return random.choice([
            lambda: "%s %s" % (adj, noun),
            lambda: "The %s %s" % (adj, noun),
            lambda: "%s of %s" % (noun, noun2),
            lambda: "%s's %s" % (noun, noun2),
            lambda: "The %s of the %s" % (noun, noun2),
            lambda: "%s in the %s" % (noun, noun2)
        ])()

    def next(self):
        return self.make_title()


""" This module uses the structured branch of the graph.
"""
from facebook.graph import GraphAPIError
from facebook.fb import TestUser, User
from facebook.testusers import TestUsers
from facebook import get_app_dict, get_static_graph
from scaffolding.tubes import Tube


class FacebookTestUser(Tube):
    def __init__(self, app_name=None, unique=False, **kwargs):
        super(FacebookTestUser, self).__init__(**kwargs)
        self.app_name = app_name
        self.unique = unique

        self.graph = get_static_graph(self.app_name)
        self.testusers = TestUsers(self.graph)
        self.users = []
        self.index = 0

    def __iter__(self):
        return self
    
    def set_up(self, cls, count, **kwargs):
        testuser_list = self.testusers.get_test_users()
        
        if count and count > len(testuser_list):
            remaining = count-len(testuser_list)
            try:
                print 'Not enough Test users (%s). Generating %s more.\n' %(len(testuser_list),
                                                                count-len(testuser_list))
            except IOError:
                pass
            for i in range(remaining):
                self._generate_new_user()
        self._user_for_testuser(testuser_list)
    
    def _user_for_testuser(self, testuser_list):
        try:
            print u'Checking for Facebook Test users...:\n'
        except IOError:
            pass
        for testuser in testuser_list:
            user, created = User.objects.get_or_create(id=int(testuser.id))
            if created:
                user.get_from_facebook(self.graph, save=True)
            self.users.append(user)
        try:
            print u'Done. Found %s testusers.' % len(testuser_list)
        except IOError:
            pass

    def _generate_new_user(self):
        newuser = self.testusers.generate_new_test_user(installed=True, permissions=['email'])
        try:
            print u'Generated new Testuser: %s\n' % newuser
        except IOError:
            pass
        # The App wants Facebook Users, not Testusers.
        user, created = User.objects.get_or_create(id=int(newuser.id))
        if created:
            user.get_from_facebook(self.graph, save=True)
        self.users.append(user)
        return user

    def next(self):
        if self.index == len(self.users)-1:
            if self.unique:
                try:
                    return self._generate_new_user()
                except GraphAPIError:
                    raise StopIteration
            else:
                self.index = 0
                return self.users[self.index]
        self.index += 1
        return self.users[self.index]


# coding=utf-8

from datetime import datetime

from django.core.management.base import BaseCommand, CommandError
from django.db import models
from django.db.models import loading

import scaffolding

import logging
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    args = '<app_label.model_name> <count>'
    help = 'Creates placeholder data for your models.'

    def handle(self, *args, **options):
        if not args or len(args) != 2:
            raise CommandError('Do: scaffold <app_label.model_name> <count>')

        app_label, separator, model_name = args[0].partition('.')
        model = loading.get_model(app_label, model_name)

        if not isinstance(model, models.base.ModelBase):
            raise CommandError('%s.%s is not a Django model.' % (app_label,
                                                                 model))

        count = int(args[1])

        self.stdout.write(u'Creating %s\n' % model)
        factory, finalizer = self.make_factory(model, count)

        for i in range(count):
            self.make_object(model, factory, finalizer)

        self.stdout.write(u'\nCreated %s %ss\n' % (count,
                                                   model._meta.model_name))


    def make_factory(self, cls, count):
        """ Get the generators from the Scaffolding class within the model.
        """
        field_names = cls._meta.get_all_field_names()
        fields = {}
        text = []
        finalizer = None
        scaffold = scaffolding.scaffold_for_model(cls)

        for field_name in field_names:
            generator = getattr(scaffold, field_name, None)
            if generator:
                if hasattr(generator, 'set_up'):
                    generator.set_up(cls, count)
                fields[field_name] = generator
                text.append(u'%s: %s; ' % (field_name, fields[field_name]))
        try:
            self.stdout.write(u'Generator for %s: %s\n' % (cls, u''.join(text)))
        except models.ObjectDoesNotExist:
            self.stdout.write(u'Generator for %s\n' % u''.join(text))

        if hasattr(scaffold, 'finalize') and hasattr(scaffold.finalize, '__call__'):
            finalizer = scaffold.finalize

        return fields, finalizer

    def make_object(self, cls, fields, finalizer=None):
        obj = cls()

        for field_name, generator in fields.items():
            # Some custom processing
            field = cls._meta.get_field(field_name)
            value = generator.next()
            if isinstance(field, models.fields.related.ForeignKey) and isinstance(value, int):
                field_name = u'%s_id' % field_name
            if isinstance(field, models.fields.files.FileField):
                getattr(obj, field_name).save(*value, save=False)
            else:
                setattr(obj, field_name, value)
            try:
                self.stdout.write(u'%s: %s; ' % (field_name, value))
            except (UnicodeEncodeError, TypeError):
                pass

        if finalizer:
            try:
                finalizer(obj)
            except Exception as e:
                self.stdout.write(u"Error finalizing Obj %s: " % e.message)
                return False

        obj.save()
        self.stdout.write(u'\nCreated new %s: ' % obj.__class__.__name__)



#!/usr/bin/env python

from setuptools import setup, find_packages
import os


setup(
    name='django-scaffolding',
    version='0.2.6',
    author='Simon Baechler',
    author_email='simon@stellanera.com',
    packages=find_packages(
        exclude=['tests', 'example']
    ),
    package_data={
        '': ['*.html', '*.txt'],
        'scaffolding': [
            'library/*.csv'
        ]
    },
    url='https://github.com/sbaechler/django-scaffolding/',
    license='MIT',
    description='Automatically generate reasonable database entries for your app',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
    ],
    zip_safe=False
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import imp
import sys

from django.conf import settings
from django.utils.importlib import import_module

from .tubes import (Tube, Name, LoremIpsum, RandInt, Contrib,
                    AlwaysTrue,
                    AlwaysFalse, StaticValue, RandomValue, EveryValue,
                    RandomInternetImage,
                    ForeignKey, FirstName, LastName, TrueOrFalse, BookTitle,
                    RandomDate, RandomDatetime,
                    ForeignKeyOrNone, USCity, URL, OrNone, OrBlank,
                    RandomEmail, Uuid
                    )

__all__ = ['Tube', 'Name', 'LoremIpsum', 'RandInt', 'Contrib', 'AlwaysTrue',
           'AlwaysFalse', 'StaticValue', 'RandomValue', 'EveryValue', 'OrNone',
           'OrBlank', 'RandomInternetImage', 'FirstName', 'LastName', 'USCity',
           'URL', 'TrueOrFalse', 'BookTitle', 'RandomDate', 'RandomDatetime', 'ForeignKeyOrNone',
           'ForeignKey', 'register', 'scaffold_for_model', 'RandomEmail',
           'Uuid']


def generic_autodiscover(module_name):

    for app in settings.INSTALLED_APPS:
        try:
            import_module(app)
            app_path = sys.modules[app].__path__
        except (AttributeError, ImportError):
            continue
        try:
            imp.find_module(module_name, app_path)
        except ImportError:
            continue
        import_module('%s.%s' % (app, module_name))
        app_path = sys.modules['%s.%s' % (app, module_name)]


_registry = {}


def register(model, scaffold):
    _registry[model] = scaffold


def scaffold_for_model(model):
    """
    Returns the scaffold class for a given model (if it has been registered
    before).

    """
    # Load scaffold modules of all INSTALLED_APPS
    generic_autodiscover('scaffolds')

    return _registry[model]



from __future__ import absolute_import, unicode_literals
import uuid
from django.db.models.query import QuerySet

import os
import random
import urllib
import datetime
import string

from django.core.files import File
from django.utils.timezone import make_aware
from scaffolding.library import lorem_ipsum
import sys


class Tube(object):
    """ The base class for scaffolding objects.
    """
    def __init__(self, **kwargs):
        pass

    def __iter__(self):
        return self

    def set_up(self, cls, count, **kwargs):
        """ This is a hook for doing validations
            kwargs for future compatibility.
        """

    def next(self):
        raise NotImplementedError('You need to implement your own next method.')

#---------- custom classes -----------------

class StaticValue(Tube):
    """Always returns the same value"""

    def __init__(self, value):
        self.value = value
    def next(self):
        return self.value


class RandomValue(Tube):
    """Returns random values from the passed list"""
    def __init__(self, lst):
        self.lst = lst
    def next(self):
        return random.choice(self.lst)


class EveryValue(Tube):
    """
    Yields values from the passed iterable in order, looping into infinity.
    """
    def __init__(self, values, **kwargs):
        self.index = -1
        self.values = list(values)
        self.length = len(self.values)

    def next(self):
        if self.length == 0:
            raise StopIteration
        self.index += 1
        return self.values[self.index % self.length]


class OrNone(Tube):
    """
    Yields values from the passed class or None.
    """
    def __init__(self, cls, split=0.5, *args, **kwargs):
        self.split = split
        self.cls = cls(*args, **kwargs)


    def next(self):
        if random.random() > self.split:
            return None
        else:
            return self.cls.next()

class OrBlank(Tube):
    """
    Yields values from the passed class or "".
    """
    def __init__(self, cls, split=0.5, *args, **kwargs):
        self.split = split
        self.cls = cls(*args, **kwargs)


    def next(self):
        if random.random() > self.split:
            return ""
        else:
            return self.cls.next()


class Name(Tube):
    """ Generates a random name. <gender> can be 'male', 'female', 'm' or 'f'.
    """
    def __init__(self, max_length=30, gender=None, **kwargs):
        from scaffolding.library import names
        super(Name, self).__init__(**kwargs)
        self.max_length = max_length
        self.first_names = names.FirstNames(gender=gender)
        self.last_names = names.LastNames()

    def next(self):
        return '%s %s'[:self.max_length] % (self.first_names.next(), self.last_names.next())

class FirstName(Name):
    """ Only returns first names. """
    def next(self):
        return '%s'[:self.max_length] % self.first_names.next()

class LastName(Name):
    """ Only returns last names. """
    def next(self):
        return '%s'[:self.max_length] % self.last_names.next()

class RandomEmail(Tube):
    """ Return a random email. """

    def __init__(self, length=8, domain="example.com"):
        self.length = length
        self.domain = domain

    def next(self):
        return ''.join(random.choice(string.ascii_lowercase)
                       for x in range(self.length)) + '@' + self.domain

class BookTitle(Tube):
    def __init__(self, **kwargs):
        from scaffolding.library import booktitles
        super(BookTitle, self).__init__(**kwargs)
        self.title = booktitles.Title()

    def next(self):
        return self.title.next()


class LoremIpsum(Tube):
    """ Generates a Lorem Ipsum Text. The number of paragraphs is defined in paragraphs.
    """
    def __init__(self, paragraphs=7, max_length=None, text=lorem_ipsum.LOREM_IPSUM, **kwargs):
        super(LoremIpsum, self).__init__(**kwargs)
        self.text = text
        self.max_length = max_length
        self.paragraphs = paragraphs
        #  TODO: Loop paragraphs.
        if self.paragraphs > len(self.text):
            raise AttributeError('The Text %s only has %s paragraphs' %(text, len(text)))

    def next(self):
        if self.paragraphs < len(self.text):
            late_start = len(self.text) - self.paragraphs - 1
            start = random.randint(0, late_start)
        else:
            start = 0
        text = '\n\n'.join(self.text[start:(start+self.paragraphs)])
        if self.max_length:
            return text[:self.max_length]
        return text


class RandInt(Tube):
    """ Generates a random integer between min and max """
    def __init__(self, min, max, **kwargs):
        super(RandInt, self).__init__(**kwargs)
        self.min = min
        self.max = max

    def next(self):
        return random.randint(self.min, self.max)


class Contrib(object):
    """ Crates a Custom Object. The backend class is the first parameter.
        The backend class has to inherit from Tube.
    """
    def __init__(self, backend, **kwargs):
        self.backend = backend(**kwargs)

    def __iter__(self):
        return self.backend

    def next(self):
        return self.backend.next()

    def set_up(self, cls, count, **kwargs):
        if hasattr(self.backend, 'set_up'):
            self.backend.set_up(cls, count, **kwargs)
        else:
            pass


class AlwaysTrue(StaticValue):
    """ Always returns True."""
    def __init__(self):
        self.value = True


class AlwaysFalse(StaticValue):
    """ Always returns False."""
    def __init__(self):
        self.value = False


class TrueOrFalse(RandomValue):
    """ Randomly returns true or false.
        You can set a ratio for true or false by specifying true and false:
        e.g. true=1, false=3 returns 3 times as many False than Trues.
    """
    def __init__(self, true=1, false=1):
        self.lst = [True for i in range(true)]
        self.lst.extend([False for i in range(false)])


class RandomInternetImage(Tube):
    """ Creates a random image for an ImageField using an internet source.
    """
    def __init__(self, backend, **kwargs):
        super(RandomInternetImage, self).__init__(**kwargs)
        self.backend = backend(**kwargs)

    def next(self):
        # returns a filename and File object, ready to be fed to the image.save() method.
        url = self.backend.next()
        temp_image = urllib.urlretrieve(url)
        return os.path.basename(url), File(open(temp_image[0]))


class ForeignKey(EveryValue):
    """ Creates a foreign key assigning items from the queryset.
    """
    def __init__(self, queryset, chunksize=100, **kwargs):
        if not isinstance(queryset, QuerySet):
            raise AttributeError("queryset needs to be an instance of a Django QuerySet."
                                 " (got a %s)" % type(queryset))
        if len(queryset) == 0:
            sys.stdout.write("Queryset for %s is empty.\n" % queryset.model)
        super(ForeignKey, self).__init__(queryset[:chunksize])


class ForeignKeyOrNone(OrNone):
    """ Maybe creates a foreign key, otherwise None.
        split is the weight for positives. 0.2 yields 80% None.
    """
    def __init__(self, **kwargs):
        super(ForeignKeyOrNone, self).__init__(cls=ForeignKey, **kwargs)


class RandomDate(Tube):
    """ Creates a date between startdate and enddate  """
    def __init__(self, startdate, enddate, **kwargs):
        super(RandomDate, self).__init__(**kwargs)
        if not (isinstance(startdate, datetime.date) and
                isinstance(enddate, datetime.date)):
            raise AttributeError(
                "startdate and enddate must be instances of datetime.date")
        if enddate < startdate:
            raise AttributeError(
                "enddate must be after startdate"
            )
        self.startdate = startdate
        self.enddate = enddate

    def next(self):
        delta = (self.enddate - self.startdate).days
        return self.startdate + datetime.timedelta(random.randint(0, delta))


class RandomDatetime(Tube):
    """ Creates a datetime between startdate and enddate. """
    def __init__(self, startdate, enddate, timezone=None, **kwargs):
        super(RandomDatetime, self).__init__(**kwargs)
        if not (isinstance(startdate, datetime.datetime) and
                isinstance(enddate, datetime.datetime)):
            raise AttributeError(
                "startdate and enddate must be instances of datetime.datetime")
        if enddate < startdate:
            raise AttributeError(
                "enddate must be after startdate"
            )
        self.startdate = startdate
        self.enddate = enddate
        self.timezone = timezone

    def next(self):
        delta = (self.enddate - self.startdate).days
        moment = self.startdate + datetime.timedelta(random.randint(0, delta))
        if self.timezone:
            moment = make_aware(moment, self.timezone)
        return moment


class USCity(RandomValue):
    def __init__(self):
        from .library.cities import TopUsCities
        top_us = TopUsCities()
        self.lst = top_us()


class URL(RandomValue):
    def __init__(self, prefix='http://'):
        from .library.url import TopUrl
        urls = TopUrl(prefix=prefix)
        self.lst = urls()


class Uuid(Tube):
    """ Generates a unique alphanumeric id """
    FORMATS = ('uuid', 'hex', 'int')
    def __init__(self, format='hex'):
        if format not in self.FORMATS:
            raise AttributeError('format %s is not valid for UUID field'
                                 % format)
        self.format = format

    def next(self):
        uid = uuid.uuid4()
        if self.format == 'uuid':
            return uid
        else:
            return getattr(uid, self.format)
